import sys

from distutils import util
from distutils import sysconfig
from distutils import log
from distutils.version import LooseVersion, StrictVersion
import glob
import os
import subprocess
import re 
from distutils.errors import DistutilsError

platform       = util.get_platform()
pythonversion  = sysconfig.get_python_version()


__saved_args__=None

def save_argv():
    __saved_args__ = list(sys.argv)

def is_python27(path=None):
    '''
    Checks that the python is a python2.7 
    
        @param path: if None consider the running python
                     otherwise the python pointed by the path
                     
        @return: True if the python is a 2.7
        @rtype: bool
    '''
    if path is None:
        pythonversion = LooseVersion(sysconfig.get_python_version())
    else:
        command = """%s -c 'from distutils import sysconfig; """ \
                  """print sysconfig.get_python_version()'""" % path
                  
        p = subprocess.Popen(command, 
                             shell=True, 
                             stdout=subprocess.PIPE)
        pythonversion = LooseVersion(p.communicate()[0])
                
    return     pythonversion >=LooseVersion("2.7") \
           and pythonversion < LooseVersion("2.8") 
           
           
           
def is_mac_system_python(path=None):
    '''
    Checks on a mac platform if the python is the original 
    python provided with the systems
    .
    
        @param path: if None consider the running python
                     otherwise the python pointed by the path
                     
        @return: True if the python is the system one
        @rtype: bool
    '''
    if path is None:
        path = sys.executable
    
    p1 = '/System/Library/Frameworks/Python.framework'
    p2 = '/usr/bin'
    
    return path[0:len(p1)]==p1 or \
           path[0:len(p2)]==p2
           
def is_mac_system():
    platform = util.get_platform().split('-')[0]
    
    return platform=='macosx'

def is_windows_system():
    platform = util.get_platform().split('-')[0]
    
    return platform=='Windows'

def is_a_virtualenv_python(path=None):
    if path is None:
        rep = hasattr(sys, 'real_prefix')
    else:
        command = """%s -c 'import sys; print hasattr(sys,"real_prefix")'""" % path
        p = subprocess.Popen(command, 
                             shell=True, 
                             stdout=subprocess.PIPE)
        rep = eval(p.communicate()[0])
        
    return rep
           
def is_good_python27(path = None):    
    rep = is_python27(path) and \
          (not is_mac_system() or \
           not is_mac_system_python(path) \
          )
    
    return rep
    
def lookfor_good_python27():
    exe = []
    if not is_windows_system():
        paths = os.environ['PATH'].split(os.pathsep)
        for p in paths:
            candidates = glob.glob(os.path.join(p,'python2.7')) + \
                         glob.glob(os.path.join(p,'python2')) + \
                         glob.glob(os.path.join(p,'python'))
            pexe = []
            for e in candidates:
                if os.path.islink(e):
                    e = os.path.realpath(e)
                if os.path.isfile(e) and \
                   os.access(e, os.X_OK) and \
                   is_good_python27(e) and \
                   not is_a_virtualenv_python(e):
                    pexe.append(e)
            exe.extend(set(pexe))
        
    return exe

def cleaned_args(args=None):
    if args is None:
        args = sys.argv
        
    setup = re.compile('setup\\.py')
    scripts = [setup.search(a) is not None for a in args]
    if any(scripts):
        args = args[scripts.index(True):]
    else:
        args = None
        
    return args

def rerun_with_anothe_python(path):
    if __saved_args__ is not None:
        args = __saved_args__
    else:
        args = sys.argv
      
    args = cleaned_args(args)
    
    assert is_good_python27(path), \
           'the selected python is not adapted to the installation of this package'
           
    prog = path.split(os.sep)[-1]

    log.info("========================================")    
    log.info("")
    log.info("    Switching to python : %s" % path)
    log.info("")
    log.info("========================================")    
        
    args = [prog]+list(args)
    
    sys.stderr.flush()
    sys.stdout.flush()
    os.execv(path,list(args))

def enforce_good_python():
    if is_good_python27():
        return True
    
    goodpython = lookfor_good_python27()
    
    if not goodpython:
        raise DistutilsError,'No good python identified on your system'

    goodpython=goodpython[0]
    
    rerun_with_anothe_python(goodpython)
    
    


    
